/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Tobias Gröger ( tobias dot groeger at student dot kit dot edu )
 * @author     Corvin Navarro Ecker ( corvin dot ecker at student dot kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <RobotAPI/libraries/ukfm/UnscentedKalmanFilter.h>

#include <armarx/navigation/human/HumanSystemModel.h>
#include <armarx/navigation/human/types.h>

namespace armarx::navigation::human
{

    class HumanFilter
    {

    public:
        struct Parameters
        {
            //TODO which values are appropriate?
            double control_pos_cov = 0.01;
            double control_rot_cov = 0.01;
            double obs_pos_cov = 0.01;
            double obs_rot_cov = 0.01;
            double initial_state_pos_cov = 0.01;
            double initial_state_rot_cov = 0.01;
            double alpha = 0.5;

            float velocityAlpha = 0.7;
        };

        HumanFilter(const core::Pose2D& pose0, const DateTime& detectionTime);

        void propagation(const DateTime& detectionTime);
        void update(const core::Pose2D& pose, const DateTime& detectionTime);

        const Human& get() const;

    private:
        using SystemModelT = kalman_filter::SystemModelSE2<double>;

        SystemModelT::StateT toUkfState(const core::Pose2D& pose);
        core::Pose2D fromUkfState(const SystemModelT::StateT& pose);
        SystemModelT::ControlT toUkfControl(const Eigen::Vector2f& vel);
        SystemModelT::ObsT toUkfObs(const core::Pose2D& pose);

    private:
        Parameters params;
        core::Pose2D oldPose;
        Human human;
        std::unique_ptr<UnscentedKalmanFilter<SystemModelT>> ukf;
    };

} // namespace armarx::navigation::human
