/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Tobias Gröger ( tobias dot groeger at student dot kit dot edu )
 * @author     Corvin Navarro Ecker ( corvin dot ecker at student dot kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <ArmarXCore/core/time.h>

#include <RobotAPI/libraries/ukfm/UnscentedKalmanFilter.h>

#include <VisionX/libraries/armem_human/types.h>

#include <armarx/navigation/core/basic_types.h>
#include <armarx/navigation/human/HumanFilter.h>
#include <armarx/navigation/human/HumanSystemModel.h>
#include <armarx/navigation/human/types.h>

namespace armarx::navigation::human
{
    using T = double; //TODO double or float?
    using Vector = Eigen::Matrix<T, 2, 1>;
    using SystemModelT = kalman_filter::SystemModelSO2xR2<T>;

    class HumanTracker
    {
    public:
        HumanTracker() = default;

        struct Measurements
        {
            DateTime detectionTime;
            std::vector<armem::human::HumanPose> humanPoses;
        };

        struct DetectedHuman
        {
            core::Pose2D pose;
            std::optional<std::string> trackingId;
            DateTime detectionTime;
            bool associated;
        };

        struct TrackedHuman
        {
            HumanFilter humanFilter;
            std::optional<std::string> trackingId = std::nullopt;
            bool associated;
        };

        struct Parameters
        {
            // the duration after which tracked humans will be erased if no new measurement for this human is found
            Duration maxTrackingAge = Duration::MilliSeconds(500);
            // the maximum distance in millimeters of two human measurements to be associated with each other
            float maxAssociationDistance = 600;
            // alpha value from interval [0,1] to determine how much the new (and respectively the old) velocity should be weighted
            float velocityAlpha = 0.7;
        };

        void update(const Measurements& measurements);

        std::vector<human::Human> getTrackedHumans() const;

        void reset();

    private:
        void associateHumans(std::vector<DetectedHuman>& detectedHumans);
        void associate(TrackedHuman* tracked, DetectedHuman* detected);

    private:
        std::vector<TrackedHuman> trackedHumans;
        Parameters parameters;
    };
} // namespace armarx::navigation::human
